# -*- coding: utf-8 -*-

import psycopg2


class PostgreDb:
    debug = False

    def __init__(self, host=None, port=None, user=None, pwd=None, db=None):
        self.host = host
        self.port = port
        self.user = user
        self.pwd = pwd
        self.db = db
        self.conn = None
        self.c = None
        if host and port and user and db:
            self.open(host=self.host, port=self.port, user=self.user, pwd=self.pwd, db=self.db)

    def open(self, host, port, user, pwd, db):
        try:
            if self.debug:
                print(host, port, user, pwd, db)
            self.conn = psycopg2.connect(host=host, port=port, user=user, password=pwd, dbname=db)
        except Exception as e:
            print(host, port, user, pwd, db)
            raise Exception(e)

        if self.conn:
            self.c = self.conn.cursor()
            return True
        else:
            return False

    def execute(self, sql):
        # 执行Sql语言。
        if self.debug:
            print("""class MysqlDb self.DEBUG:\t execute: %s""" % str(sql))
        try:
            self.c.execute(sql)
            return True
        except Exception as e:
            if self.debug:
                print("""class MysqlDb self.DEBUG:\t ERROR: %s""" % str(e))
            return False

    def fetchall(self):

        result = self.c.fetchall()
        if self.debug:
            print(f"fetchall {result}")
        return result

    def fetchone(self):

        result = self.c.fetchone()
        if self.debug:
            print(f"fetchone {result}")
        return result

    def commit(self):
        if self.conn:
            self.conn.commit()

    def close(self):
        if self.conn:
            self.conn.close()



# -*- coding: utf-8 -*-

import xml.etree.ElementTree as ET
import os
from .utils import SqliteDb, MySqlDb, PostgreDb

DB_TYPES = ['mysqlin', 'mysql', 'pgsql']


def xml_replace(xml_path: str):
    """
    xml文件中特殊字符的替换，XML有标准字符，如&不可以出现。使用前要做处理。
    注意XML标准不允许使用&符号，Winmail配置中有可能出现，必须事先处理。
    将&符号替换为(|and|)，在返回结果时再把"(|and|)"替换成&。
    """
    xml = ""
    with open(xml_path, 'r', encoding="utf-8") as fd:
        for line in fd:
            if "&" in line:
                line = line.replace("&", "(|and|)")
            xml += line

    return xml


def xml_unreplace(s):
    """
    把xmlReplace()替换的字符替换回去。
    """
    result = s
    if "(|and|)" in s:
        result = s.replace("(|and|)", "&")
    return result


def get_xml_element_value(xml_path, element_path):
    """
    使用ET的查找返回element节点。返回结果
    注意XML标准不允许使用&符号，Winmail的system.cfg配置中有可能出现，在读取之前必须事先处理。
    将&符号替换为(|and|)，在返回结果时再把"(|and|)"替换成&。
    使用函数xmlReplace()xmlUnReplace()处理。

    :xml_path: 配置文件完整路径
    :element_path: XML路径
    :return:
    """

    xml = xml_replace(xml_path)

    root = ET.fromstring(xml)
    it = root.find(element_path)
    # print(f'{xml_path}: {element_path}-> {it.text}')
    if it is None:
        return ''
    else:
        if it.text is None:
            return ''
        return xml_unreplace(it.text)


def get_all_xml_element_value(xml_path, element_path):
    """
    查询配置文件中的有重复的项比如 ".//smtpoption//domaindelivery//domainhost//"
    """
    result = []
    xml = xml_replace(xml_path)

    root = ET.fromstring(xml)

    for it in root.findall(element_path):
        # print(it.tag, it.attrib, it.text)
        result.append(xml_unreplace(it.text))
    return result


def modify_xml_element(xml_path, element_path, value_text):
    """
    使用ET的findall查找返回element节点。直接给节点赋值返回bool。

    :xml_path: xml文件完整路径
    :element_path: 节点完整路径
    :value_text:节点值
    :return:
        bool
    """

    tree = ET.parse(xml_path)

    root = tree.getroot()

    for it in root.findall(element_path):
        # print(iter.text)
        it.text = value_text
    tree.write(xml_path, encoding="utf-8")
    for it in root.findall(element_path):
        if it.text != value_text:
            return False

    return True


def insert_xml_element(xml_path, element_path, element_target, element_text):
    """

    :xmlPath: XML文件路径
    :elementPath: 要插入到的节点
    :elementTarget: 要插入的节点名
    :elementText: 要插入的节点值
    :return:
        bool.
    """
    tree = ET.parse(xml_path)
    root = tree.getroot()

    # 使用ET.Element("elementTarget") 把节点先建好。
    node = ET.Element(element_target)
    node.text = "\n"
    node.text = element_text
    # 使用root.iter，打开节点，循环打开elementPath，直到最后一层，再使用append插入。
    path_list = []
    for i in element_path.split("/"):
        if i != "":
            path_list.append(i)

    n = len(path_list) - 1
    m = 0
    for i in path_list:

        if m == 0 and (root.tag == path_list[0] or path_list[0] == "."):
            m += 1
            continue

        iter_tag = ""
        for it in root.iter(i):
            if ET.iselement(it):
                iter_tag = it.tag
                it.text = "\\n" + it.text

        if iter_tag == i:
            if m == n:
                # insert node插入切点位置
                for it in root.iter(i):
                    it.append(node)
                    tree.write(xml_path, encoding="utf-8")
                return True
            m += 1
            continue
        else:
            return False


def get_winmail_path():
    """
    获取Winmail的安装路径
    """

    winmail_path = None
    error_info = 'os.name check error!'

    if os.name == "nt":
        # 优先使用注册表取路径，如果失败从DLL取，DLL有32位64位区分，如果要打包，需要在不同环境分别打包。
        import winreg

        try:
            regkey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                                    "SYSTEM\\ControlSet001\\services\\MagicWinmailServer")
            winmail_path = os.path.dirname(
                winreg.QueryValueEx(regkey, "ImagePath")[0].split('\"')[1])  # ~Magic Winmail\\server\\
        except OSError as e:
            error_info = [f"Get winreg "
                          f"HKEY_LOCAL_MACHINE\\SYSTEM\\ControlSet001\\services\\MagicWinmailServer"
                          f" error! see: {str(e)}"]

        if not winmail_path:
            try:
                regkey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                                        "SYSTEM\\CurrentControlSet\\services\\MagicWinmailServer")
                winmail_path = os.path.dirname(
                    winreg.QueryValueEx(regkey, "ImagePath")[0].split('\"')[1])  # ~Magic Winmail\\server\\
            except Exception as e:
                error_info.append(f"Get winreg "
                                  f"HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\services\\MagicWinmailServer"
                                  f" error! see: {str(e)}")
        if not winmail_path:
            try:
                from win32com.client import Dispatch

                dll = Dispatch("MailServerCtrl.MailDBInterface")
                dll.InitControl("")
                winmail_path = dll.GetDBPath()
            except Exception as e:
                error_info.append(f"Dispatch(\"MailServerCtrl.MailDBInterface\") error:{str(e)}")

    if os.name == "posix":
        from configparser import ConfigParser

        service_file = "/usr/lib/systemd/system/winmail.service"
        if os.path.isfile(service_file):
            winmail_service = ConfigParser()
            winmail_service.read(service_file)
            winmail_path = winmail_service['Service']['WorkingDirectory']
            error_info = 'Get Winmail Path from /usr/lib/systemd/system/winmail.service error!'
        if not winmail_path:
            winmail_path = "/opt/winmail"
            error_info = 'Get Winmail Path error!'

    if os.path.isdir(winmail_path):
        return winmail_path
    else:
        raise Exception(error_info)


class Winmail:
    """
    获取各Winmail相关的路径
    建议使用OpenAPI接口，此方法直接读取后台配置文件和数据库。对版本符合度要求高。测试版本为Winmail 6.7、7.0
    """

    def __init__(self, db_pwd=None):

        self.winmail_path = get_winmail_path()

        self.winmail_sys_conf_path = os.path.join(self.winmail_path, "data//system.cfg")
        self.winmail_domain_conf_path = os.path.join(self.winmail_path, "data//domain.db")
        self.winmail_mailgroup_conf_path = os.path.join(self.winmail_path, "data//mailgroup.db")
        self.winmail_mailuser_stat_conf_path = os.path.join(self.winmail_path, "data//mailuserstat.db")
        self.winmail_mailuser_conf_path = os.path.join(self.winmail_path, "data//mailuser.db")

        if not self.check_winmail_path():
            raise Exception("Get WinmailPath Error : self.winmailPath -> %s;" % self.winmail_path)

        self.db_type = self.get_db_type()
        self.db_pwd = db_pwd
        self.tblprefix = ''
        self.domain_table = 'domain'
        self.domainalias_table = 'domainalias'
        self.mailuser_table = 'mailuser'
        self.useralias_table = 'useralias'
        self.mailgroup_table = 'mailgroup'
        self.mailuser_stat_table = 'stat'

        # 如非sqlite数据库，从配置中取前缀并定义数据库表名，并取出数据库连接相关信息。系统自带的mysqlin使用root用户。密码在管理工具中显示。
        if self.db_type in DB_TYPES:
            self.tblprefix = get_xml_element_value(self.winmail_sys_conf_path, f'./dboption/{self.db_type}/tblprefix')

            self.domain_table = self.tblprefix + 'domain'
            self.domainalias_table = self.tblprefix + 'domainalias'
            self.mailuser_table = self.tblprefix + 'mailuser'
            self.useralias_table = self.tblprefix + 'useralias'
            self.mailgroup_table = self.tblprefix + 'mailgroup'
            self.mailuser_stat_table = self.tblprefix + 'mailuserstat'

            self.db_host = get_xml_element_value(self.winmail_sys_conf_path, f'./dboption/{self.db_type}/host')
            self.db_port = int(get_xml_element_value(self.winmail_sys_conf_path, f'./dboption/{self.db_type}/port'))
            if self.db_type == 'mysqlin':
                self.db_user = 'root'
            else:
                self.db_user = get_xml_element_value(self.winmail_sys_conf_path, f'./dboption/{self.db_type}/username')
            self.db_database = get_xml_element_value(self.winmail_sys_conf_path, f'./dboption/{self.db_type}/database')

        if self.db_type and self.db_type != 'sqlite' and self.db_type not in DB_TYPES:
            raise Exception("Get Winmail Database Type Error : self.db_type -> %s;" % self.db_type)

    def connect_db(self, sqlite_db_path: str = None):
        """
        按数据库类型打开数据库

        :return: Object
        """

        if self.db_type == 'mysql':

            return MySqlDb(host=self.db_host, port=self.db_port,
                           user=self.db_user, pwd=self.db_pwd,
                           db=self.db_database)

        if self.db_type == 'mysqlin':

            return MySqlDb(host=self.db_host, port=self.db_port,
                           user=self.db_user, pwd=self.db_pwd,
                           db=self.db_database)

        if self.db_type == 'pgsql':

            return PostgreDb(host=self.db_host, port=self.db_port,
                             user=self.db_user, pwd=self.db_pwd,
                             db=self.db_database)

        if sqlite_db_path:
            return SqliteDb(sqlite_db_path)
        else:
            return SqliteDb()

    def get_sys_config(self, element_path):

        if os.path.isfile(self.winmail_sys_conf_path):
            value_string = get_xml_element_value(self.winmail_sys_conf_path, element_path)
            return value_string
        return None

    def get_db_type(self):

        db_type_element_path = "./dboption/dbtype"

        return self.get_sys_config(db_type_element_path)

    def check_winmail_path(self):
        """
        winmailPath:Winmail安装目录路径，要求指向到~magic Winmail\\server\\
        :return: bool
        """

        if os.path.isdir(self.winmail_path):
            if os.path.isfile(self.winmail_mailgroup_conf_path) \
                    and os.path.isfile(self.winmail_mailuser_stat_conf_path) \
                    and os.path.isfile(self.winmail_sys_conf_path) \
                    and os.path.isfile(self.winmail_domain_conf_path) \
                    and os.path.isfile(self.winmail_mailuser_conf_path):
                return True
        return False

    def get_user_mailstore_path(self, email):
        """
        获取用户的邮件存储路径
        :param email: 用户邮件地址
        :return: string。 路径
        """

        user, domain = email.split("@")
        mailstore_path = ""
        store_element_path = "./advanced/directory/mailstore"

        if self.check_pri_domain(domain):
            user_db_domain = ""
        else:
            user_db_domain = domain

        # 查询用户属性中的mailstorepath。

        user_sql = \
            f'select mailstorepath from {self.mailuser_table} where domain=\'{user_db_domain}\' and name=\'{user}\';'

        db_handel = self.connect_db()

        if self.db_type not in DB_TYPES:
            db_handel = self.connect_db(sqlite_db_path=self.winmail_mailuser_conf_path)

        db_handel.execute(user_sql)
        sql_result = db_handel.fetchone()
        db_handel.close()

        if isinstance(sql_result, tuple) and sql_result[0]:
            mailstore_path = sql_result[0]

        # check mailstorepath from domain.db.
        domain_sql = f"select mailstorepath from {self.domain_table} where domain=\'{domain}\';"
        # 未在用户表中查到路径时，在域名属性中检查是否有配置域的存储路径
        if not mailstore_path:

            db_handel = self.connect_db()
            if self.db_type not in DB_TYPES:
                db_handel = self.connect_db(sqlite_db_path=self.winmail_mailuser_conf_path)

            db_handel.execute(domain_sql)
            sql_result = db_handel.fetchone()
            db_handel.close()

            if isinstance(sql_result, tuple) and sql_result[0]:

                if user_db_domain:
                    mailstore_path = os.path.join(sql_result[0], f"{user}@{domain}")
                else:
                    mailstore_path = os.path.join(sql_result[0], f"{user}")

        # 未在用户和域属性中的配置路径下取system.cfg配置文件数据
        if not mailstore_path:

            mailstore_path = get_xml_element_value(self.winmail_sys_conf_path, store_element_path)

            # 如果配置文件中查到结果
            if mailstore_path:
                if user_db_domain:
                    mailstore_path += f"//{user}@{domain}"
                else:
                    mailstore_path += f"//{user}"
            # 结果为None或者为空字符时取系统安装目录
            else:
                if user_db_domain:
                    mailstore_path = self.winmail_path + f"//store//{user}@{domain}"
                else:
                    mailstore_path = self.winmail_path + f"//store//{user}"

        return mailstore_path

    def get_user_netstore_path(self, email):
        """
        获取用户的网盘路径
        :param email: 用户邮件地址
        :return: 字符串路径
        """

        user, domain = email.split("@")
        mailstore_path = ""
        store_element_path = "./advanced/directory/netstore"

        # 确定主域名是什么，mailuser.db中的主域查询是使用空字符串，普通域使用正常域名。
        if self.check_pri_domain(domain):
            user_db_domain = ""
        else:
            user_db_domain = domain

        # 查询用户属性中的mailstorepath。
        user_sql = \
            f'select ftpstorepath from {self.mailuser_table} where domain=\'{user_db_domain}\' and name=\'{user}\';'

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_mailuser_conf_path)

        db_handel.execute(user_sql)
        sql_result = db_handel.fetchone()
        db_handel.close()

        if isinstance(sql_result, tuple) and sql_result[0]:
            mailstore_path = sql_result[0]
        # 查询域名中的指定路径
        domain_sql = f"select ftpstorepath from {self.domain_table} where domain=\'{domain}\';"

        if not mailstore_path:

            db_handel = self.connect_db()
            if self.db_type not in DB_TYPES:
                db_handel.open(self.winmail_domain_conf_path)

            db_handel.execute(domain_sql)
            sql_result = db_handel.fetchone()
            db_handel.close()

            if isinstance(sql_result, tuple) and sql_result[0]:

                if user_db_domain:
                    mailstore_path = os.path.join(sql_result[0], f"{user}@{domain}")
                else:
                    mailstore_path = os.path.join(sql_result[0], f"{user}")

        # 查询system配置文件中路径
        if not mailstore_path:

            mailstore_path = get_xml_element_value(self.winmail_sys_conf_path, store_element_path)
            if mailstore_path:

                if user_db_domain:
                    mailstore_path += f"//{user}@{domain}"
                else:
                    mailstore_path += f"//{user}"
            else:

                if user_db_domain:
                    mailstore_path = self.winmail_path + f"//netstore//{user}@{domain}"
                else:
                    mailstore_path = self.winmail_path + f"//netstore//{user}"

        return mailstore_path

    def get_domain_mailstore_path(self, domain):
        """
        获取域名的邮件存储目录
        :param domain: 域名
        :return: 字符串路径
        """
        mailstore_path = ""
        store_element_path = "./advanced/directory/mailstore"

        domain_sql = f'select mailstorepath from {self.domain_table} where domain=\'{domain}\';'

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_domain_conf_path)

        db_handel.execute(domain_sql)
        sql_result = db_handel.fetchone()
        db_handel.close()

        if isinstance(sql_result, tuple) and sql_result[0]:
            mailstore_path = sql_result[0]

        # 域名属性中没有配置时取system.cfg配置。
        if not mailstore_path:

            mailstore_path = get_xml_element_value(self.winmail_sys_conf_path, store_element_path)
            if not mailstore_path:
                mailstore_path = os.path.join(self.winmail_path, "store")

        return mailstore_path

    def get_domain_netstore_path(self, domain):
        """
        获取域名的网盘路径
        """
        mailstore_path = ""
        store_element_path = "./advanced/directory/netstore"

        domain_sql = f'select ftpstorepath from {self.domain_table} where domain=\'{domain}\';'

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_domain_conf_path)

        db_handel.execute(domain_sql)
        sql_result = db_handel.fetchone()
        db_handel.close()

        if isinstance(sql_result, tuple) and sql_result[0]:
            mailstore_path = sql_result[0]

        if not mailstore_path:

            mailstore_path = get_xml_element_value(self.winmail_sys_conf_path, store_element_path)

            if not mailstore_path:
                mailstore_path = os.path.join(self.winmail_path, "netstore")

        return mailstore_path

    def get_mailbackup_path(self):
        """
        获取Winmail邮件备份路径
        """
        backup_element_path = ".//advanced//directory//mailbackup"

        if os.path.isfile(self.winmail_sys_conf_path):
            mail_backup_path = get_xml_element_value(self.winmail_sys_conf_path, backup_element_path)
            if mail_backup_path is None or mail_backup_path == '':
                mail_backup_path = os.path.join(self.winmail_path, "backup")
            return mail_backup_path
        return None

    def get_mailarchive_path(self):
        """
        获取winmail归档存储路径
        """
        archive_element_path = ".//advanced//directory//mailarchive"

        if os.path.isfile(self.winmail_sys_conf_path):
            mail_archive_path = get_xml_element_value(self.winmail_sys_conf_path, archive_element_path)
            # print(mail_archive_path)
            if mail_archive_path is None or mail_archive_path == '':
                mail_archive_path = os.path.join(self.winmail_path, "archive")
            return mail_archive_path
        return None

    def get_pri_domain(self):
        """
        返回结果包含域名和域别名中的已存在的域名
        :return: str domain域名
        """

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_domain_conf_path)

        db_handel.execute(f'select domain from {self.domain_table} where type=\'1\';')
        domain = db_handel.fetchone()[0]
        db_handel.close()
        return domain

    def get_domain_list(self):
        """
        返回结果包含域名域名
        :return:  list [domainName]域名
        """
        domain_list = []

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_domain_conf_path)

        db_handel.execute(f'select domain from {self.domain_table};')
        sql_result = db_handel.fetchall()
        if sql_result:
            for domain in sql_result:
                domain_list.append(domain[0])
        db_handel.close()
        return domain_list

    def get_all_domain_list(self):
        """
        返回结果包含域名和域别名中的已存在的域名
        :return:  list [domainName]域名
        """
        domain_list = []

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_domain_conf_path)

        db_handel.execute(f'select domain from {self.domain_table} union select domain from {self.domainalias_table};')
        sql_result = db_handel.fetchall()
        if sql_result:
            for domain in sql_result:
                domain_list.append(domain[0])
        db_handel.close()

        return domain_list

    def get_domain_tuple_list(self):
        """
        获取域名tuple组成的list
        :return:    tuple组成的list [(domainName,domainType)]域名,域类型（是否主域）
        """

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_domain_conf_path)
        db_handel.execute(f'select domain,type from {self.domain_table};')
        domain_list = db_handel.fetchall()
        db_handel.close()

        return domain_list

    def get_user_list(self):
        """
        获取用户列表
        :return:   list，[[email,fullname][email,fullname]]
        """

        pri_domain = ""

        domain_list = self.get_domain_tuple_list()
        if domain_list:
            for dl in domain_list:
                if dl[1] == 1:
                    pri_domain = dl[0]

        new_user_list = []

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_mailuser_conf_path)

        db_handel.execute(f'select name,domain,fullname from {self.mailuser_table};')
        user_list = db_handel.fetchall()
        db_handel.close()
        if user_list:
            for li in user_list:
                if li[1] == "":
                    new_user_list.append([li[0] + "@" + pri_domain, li[2]])
                else:
                    new_user_list.append([li[0] + "@" + li[1], li[2]])

        return new_user_list

    def get_user_alias_list(self):
        """
        获取用户别名列表。
        :return:   list，[[email,realuser],[email,realuser]]
        """
        pri_domain = ""

        domain_list = self.get_domain_tuple_list()
        if domain_list:
            for dl in domain_list:
                if dl[1] == 1:
                    pri_domain = dl[0]

        new_user_list = []

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_mailuser_conf_path)

        db_handel.execute(f'select name,domain,realuser from {self.useralias_table};')
        user_list = db_handel.fetchall()
        db_handel.close()
        if user_list:
            for li in user_list:
                if li[1] == "":
                    new_user_list.append([li[0] + "@" + pri_domain, li[2]])
                else:
                    new_user_list.append([li[0] + "@" + li[1], li[2]])
        return new_user_list

    def get_group_list(self):
        """
        获取组列表
        :return:  list[email,email]
        """
        pri_domain = ""

        domain_list = self.get_domain_tuple_list()
        if domain_list:
            for dl in domain_list:
                if dl[1] == 1:
                    pri_domain = dl[0]

        new_user_list = []

        db_handel = self.connect_db()
        if self.db_type not in DB_TYPES:
            db_handel.open(self.winmail_mailgroup_conf_path)
        db_handel.execute(f'select name,domain from {self.mailgroup_table};')
        user_list = db_handel.fetchall()
        db_handel.close()
        if user_list:
            for li in user_list:
                if li[1] == "":
                    new_user_list.append(li[0] + "@" + pri_domain)
                else:
                    new_user_list.append(li[0] + "@" + li[1])
        return new_user_list

    def check_pri_domain(self, domain):
        """
        通过Domain和tuple组成的list [(domainName:domainType)]域名:域类型（是否主域)对比。
        :param domain: 域名
        :return: bool
        """
        for i in self.get_domain_tuple_list():
            if i[1] == 1:
                if i[0] == domain:
                    return True
        return False


if __name__ == "__main__":
    wm = Winmail('123456')
    print(wm.db_type)
    print(wm.domain_table, wm.domainalias_table, wm.useralias_table, wm.mailuser_table, wm.mailgroup_table,
          wm.mailuser_stat_table)
    print(wm.get_user_mailstore_path('a@test.com'))
    print(wm.get_user_list())
    print(wm.get_pri_domain())
    print(wm.get_domain_list())
    print(wm.get_all_domain_list())
    print(wm.get_user_alias_list())

    print(wm.get_group_list())

    print(wm.get_mailbackup_path())
    print(wm.get_mailarchive_path())

    print("domain mailstore", wm.get_domain_mailstore_path("winmail.cn"))
    print("domain netstore", wm.get_domain_netstore_path("winmail.cn"))

    print("user mailstore", wm.get_user_mailstore_path("a@winmail.cn"))
    print("user netstore", wm.get_user_netstore_path("a@winmail.cn"))

    print(os.path.join(wm.get_domain_mailstore_path('winmail.cn'), 'AAAAA'))



# -*- coding:utf-8 -*-

from typing import Optional, Dict, Any
import requests
import hashlib
import time
import os
import copy


class OpenApi:
    """
    api = WinmailApi(server, port, apikey, apisecret, use_ssl=False)

    # admin login
    login_result = api.login('adminUser', 'myPassword', manage_path='admin')

    method_params = {
        "method": "user",
        "domain": "test.com"
    }

    method_result = api.get_api(**method_params)


    # user login pc: tid=2, mobile: tid=6
    login_result = api.login('adminUser', 'myPassword', tid=2)

    method_params = {
        "method": "msglist",
    }

    method_result = api.get_api(**method_params)
    """

    def __init__(self, server: str, port: int,
                 apikey: str, apisecret: str,
                 use_ssl: Optional[bool] = False) -> None:
        """初始化

        初始化接口

        :param str server: 服务器地址
        :param int port: 服务器web端口
        :param str apikey: OpenApi key
        :param str apisecret: _OpenApi secret
        :param bool use_ssl: 是否使用HTTPS, defaults to False
        """
        self.sessid: str = ''
        self.url: str = ''
        self.server: str = server
        self.port: int = port
        self.use_ssl: bool = use_ssl
        self.apikey: str = apikey
        self.apisecret: str = apisecret
        self.params: Dict[str, Any] = {}

    def sign_para(self, params: Dict[str, Any]) -> Dict[str, Any]:
        """请求参数签名

        字典参数排序，计算sign，返回对应的新字典参数
        :param params:
        :return: 返回带sign的新字典参数
        """

        sort_params = sorted(params.items(), key=lambda t: t[0], reverse=False, )
        # {'b':2,'a':1,'c':3} -> [('a', 1), ('b', 2), ('c', 3)]

        urlstr = self.apisecret
        for item in sort_params:
            for x in item:
                urlstr += x
        urlstr += self.apisecret
        sign = hashlib.md5(urlstr.encode('utf-8')).hexdigest()

        params.update({"sign": sign})
        
        self.params = {}
        self.params = params

        return params

    # http request get json data ,return dict
    def request(self, method_params: Dict[str, Any]) -> Dict[str, Any]:
        """ http/https请求

        Winmail 6.6的上传附件接口处理，sign中不计算attachfile
        :param method_params: OpenAPI请求参数字典
        :return:
        """

        if 'attachfile' in method_params:
            attch_file = method_params['attachfile']
            if isinstance(attch_file, list):
                files = []
                for f in attch_file:
                    fname = os.path.split(f)[1]
                    files.append(('attachfile[]', (fname, open(f, 'rb'))))
            else:
                files = {'attachfile': open(attch_file, 'rb')}

            method_params.pop('attachfile')

            try:
                response = requests.post(self.url, self.sign_para(method_params), files=files, verify=False)
            except Exception as e:
                return {'result': 'error', 'info': 'http connect error 0 : %s' % e}
        else:

            try:
                response = requests.post(self.url, self.sign_para(method_params), verify=False)
            except Exception as e:
                return {'result': 'error', 'info': 'http connect error 1 : %s' % e}

        if response.status_code == 200:
            # Winmail 本身newmsg.reset接口没有返回json数据，执行后就完成。单独处理下
            # (6.6 0812版本已经加入json返回结果, 注释掉)
            # if method_params['method'] == 'newmsg.reset':
            #    return {"result": 'ok'}
            try:
                data = response.json()
            except Exception as e:
                return {'result': 'error', 'info': '%s : %s' % (method_params['method'], e)}
        else:
            data = {"result": "error", 'info': "http status %s." % response.status_code}

        return data

    # login 
    def login(self, user: str, pwd: str, manage_path: Optional[str] = '', tid: Optional[int] = 0):
        """认证

        登陆成功将把sessid赋值给实例的sessid。
        返回结果为API接口返回的JSON字符串转换的dict。

        :param str user: 用户名
        :param str pwd: 密码
        :param str manage_path: 管理端路径默认是admin，6.6版本的Winmail可以在管理工具后台修改管理端路径。
                如果值为空是邮箱用户。
        :param int tid: Webmail的themes，tid=6时为手机web界面
        :return: dict {result: ok/error, info: ""}
        """

        # HTTPS选择
        if self.use_ssl:
            url_scheme = 'https://'
        else:
            url_scheme = 'http://'

        if manage_path:
            self.url = url_scheme + self.server + ':' + str(self.port) + '/' + manage_path + '/openapi.php'
        else:
            self.url = url_scheme + self.server + ':' + str(self.port) + '/openapi.php'

        # method=login
        timestamp = str(int(time.time()))
        param_dict = {'user': user, 'pass': pwd}
        param_dict.update({"apikey": self.apikey, "method": "login", "timestamp": timestamp})

        if not manage_path:
            param_dict.update({'tid': str(tid)})

        result = self.request(param_dict)
        if result['result'] == 'ok':
            self.sessid = result['info']['sessid']
        return result

    # update session id
    def update_session(self):
        """ 更新Sessid

        :return: True/False
        """
        if not self.sessid:
            return False
        timestamp = str(int(time.time()))
        param_dict = {"apikey": self.apikey, "method": "updatesession", "sessid": self.sessid, "timestamp": timestamp}
        result = self.request(param_dict)

        if result['result'] == 'ok':
            return True
        return False

    # method function return dict data
    def get_api(self, **kwargs: Dict[str, Any]) -> Dict[str, Any]:
        """接口请求

        执行OpenAPI请求，并返回rquests执行结果。

        :param dict kwargs: OpenApi方法和参数集合的字典 {'method': 'user', 'domain': 'winmail.cn' .....}

        :return: JSON {result: ok/error, info: ""}
        """

        param_dict: Dict[str, Any] = kwargs

        if not self.url or not self.sessid:

            self_name = self.__class__.__name__

            for name, obj in globals().items():
                if obj is self:
                    self_name = repr(name)

            return {"result": "error",
                    "info": "Use login first %s.login(user, pwd, manage_path) " % self_name}

        timestamp = str(int(time.time()))

        if 'sessid' not in param_dict:
            param_dict.update({'sessid': self.sessid})
        if 'apikey' not in param_dict:
            param_dict.update({"apikey": self.apikey})
        if 'timestamp' not in param_dict:
            param_dict.update({"timestamp": timestamp})
        result = self.request(param_dict)

        return result



# -*- coding: utf-8 -*-

from win32com.client import Dispatch


class ComApi:
    """ Com接口说明

    from winmail import ComApi as WinmailCom

    wc = WinmailCom()
    winmail_db_path = wc.get_db_path()
    add_domain = wc.add_domain('test.com')
    print(add_domain)
    """

    def __init__(self) -> None:
        self.dll = Dispatch("MailServerCtrl.MailDBInterface")
        # init COM
        self.dll.InitControl("")
        self.winmail_path = self.dll.GetDBPath()

    def get_db_path(self) -> str:
        # Get Winmail path
        winmail_path = self.dll.GetDBPath()
        return winmail_path

    def add_domain(self, strDomain) -> int:
        return self.dll.AddDomain(strDomain)

    def modify_domain(self,
                      strDomain,
                      strNTDomain,
                      strHost,
                      strIP,
                      strAdmin,
                      uRegister,
                      uConfirm) -> int:
        return self.dll.ModifyDomain(strDomain, strNTDomain, strHost, strIP, strAdmin, uRegister, uConfirm)

    def modify_domain_right(self,
                            strDomain,
                            lExpireTime,
                            uPop3Control,
                            uImapControl,
                            uWebmailControl,
                            uNetStoreControl,
                            uCalendarControl,
                            uNotebookControl,
                            uExternalPop3
                            ) -> int:
        return self.dll.ModifyDomainRight(strDomain, lExpireTime, uPop3Control, uImapControl, uWebmailControl,
                                          uNetStoreControl, uCalendarControl, uNotebookControl, uExternalPop3)

    def delete_domain(self, strDomain) -> int:
        return self.dll.DeleteDomain(strDomain)

    def add_user(self, strDomain, strUser, strPassword) -> int:
        return self.dll.AddUser(strUser, strDomain, strPassword)

    def modify_user_quota(self,
                          strUser,
                          strDomain,
                          lMailQuota,
                          iMailTotalLimit,
                          iWarningLimit,
                          lNetStoreQuota,
                          iNetStoreTotalLimit
                          ) -> int:
        return self.dll.ModifyUserQuota(strUser, strDomain, lMailQuota, iMailTotalLimit, iWarningLimit, lNetStoreQuota,
                                        iNetStoreTotalLimit)

    def delete_user(self, strUser, strDomain) -> int:
        return self.dll.DeleteUser(strUser, strDomain)

    def check_user(self, strUser, strDomain):
        return self.dll.CheckUser(strUser, strDomain)

    def modify_domain(self,
                      strDomain,
                      strNTDomain,
                      strHost,
                      strIP,
                      strAdmin,
                      uRegister,
                      uConfirm
                      ) -> int:
        return self.dll.ModifyDomain(strDomain, strNTDomain, strHost, strIP, strAdmin, uRegister, uConfirm)

    def modify_user(self,
                    strUser,
                    strDomain,
                    strFullName,
                    strDescription,
                    strCompany,
                    strOffice
                    ) -> int:
        return self.dll.ModifyUser(strUser, strDomain, strFullName, strDescription, strCompany, strOffice)

    def modify_user1(self,
                     pstrUser,
                     pstrDomain,
                     pstrMobile,
                     pstrHomePhone,
                     pstrHomeAddress,
                     pstrJobTitle,
                     pstrOfficePhone) -> int:
        return self.dll.ModifyUser1(pstrUser, pstrDomain, pstrMobile, pstrHomePhone, pstrHomeAddress, pstrJobTitle,
                                    pstrOfficePhone)



# -*- coding:utf-8 -*-

from .OpenApi import OpenApi, Base, OpenApi12
from .Winmail import Winmail
from .utils import start_winmail_service, stop_winmail_service

__all__ = ["OpenApi", "OpenApi12", "Base", "Winmail", "start_winmail_service", "stop_winmail_service"]

import os
if os.name == "nt":
    from .ComApi import ComApi
    
    __all__.append("ComApi")



# -*- coding:utf-8 -*-
from enum import Enum
from typing import Dict, Optional, Any, Union
from pydantic import Field
from winmail.OpenApi import OpenApi


class AuthType(str, Enum):
    """用户认证类型

    internal = 内部认证
    nt = NT认证
    third_party = 第三方认证
    """
    internal = '0'
    nt = '1'
    third_party = '2'


class UserStatus(str, Enum):
    """用户状态

    normal = 正常
    disabled = 禁止
    pending = 待审核
    paused = 暂停
    """

    normal = '0'
    disabled = '1'
    pending = '2'
    paused = '3'


class Base(OpenApi):

    def __init__(self, server: str, port: int, apikey: str, apisecret: str, use_ssl: bool = False) -> None:
        """
        初始化OpenApi接口

        :param server: 服务器地址
        :param port:  端口
        :param apikey: APIkey
        :param apisecret: API密钥
        :param use_ssl:  是否使用SSL端口
        :return: None
        """
        super().__init__(server, port, apikey, apisecret, use_ssl)
        self.login_status = False
        self.login_result = None

    # #####通用接口开始
    def login(self, user: str, pwd: str, manage_path: Optional[str] = '', tid: int = 0) -> Dict[str, Any]:
        """ 登陆认证

        登陆并修改登陆状态login_status,保存sessid。

        :param user: 认证邮箱用户名
        :param pwd:  密码
        :param manage_path: 调用管理端接口请写入管理地址，用户端调用请忽略，【winmail管理工具中系统设置-高级设置-系统参数-HTTP配置中查看】
        :param tid: 用户端调用参数，手机风格6，否则默认为0。参考web代码中用户登陆风格。
        :return: Dict
            管理端返回：
            {
                "result": "ok",
                "info":
                {
                    “ sessid ”:  "efd24f9f63d69d6f6f169b235822ca5875eb2bae",
                    “ user ”:  "admin"
                }
            }
            用户端返回：
            {
            "result": "ok",
            "info":
                {
                “ sessid ”:  "1a2db1b0edda735295508723d8c6a962",
                “uid”: "test",
                “email”: " test@cnu.com ",
                “fullname”: "测试用户",
                “mobile”: "13900000000",
                “company”: "华兆科技",
                “department”: "系统研发部",
                “jobtitle”: "工程帅",
                “office”: "",
                “officephone”: "",
                “homeaddress”: "",
                “homephone”: "",
                }
            }
        """

        self.login_result = super().login(user, pwd, manage_path=manage_path, tid=tid)

        if self.login_result['result'] == 'ok':
            self.login_status = True

        return self.login_result

    def updatesession(self):
        """更新会话

        更新Sessid，sessid默认为30分钟超时。
        :return: bool
        """
        return self.update_session()

    # #####管理端接口开始
    def domain(self, pageno: Optional[int] = 0) -> Dict[str, Any]:
        """ 获取域名

        取所有域名列表

        :param pageno: 分页序号，以 0 开始
        :return: Dict
            {
            "result": "ok",
            "info":
                {
                “ domains ”: 域名列表 ,
                “ totalcount ”: 域名总数
                “ pagecount ”:  分页总数
                }
            }
        """

        method_params = {"sessid": self.sessid, "method": "domain"}
        method_params.update({"pageno": str(pageno)})

        return self.get_api(**method_params)

    def domain_added(self, domain: str,
                     description: Optional[str] = None,
                     mailquota: Optional[int] = None,
                     mailcount: Optional[int] = None,
                     ftpquota: Optional[int] = None,
                     ftpcount: Optional[int] = None,
                     **kwargs: Optional[Dict]
                     ) -> Dict[str, Any]:
        """ 新增域名

        新增域名接口

        :param domain: 域名
        :param description: 描述
        :param mailquota: 新邮箱默认空间大小,单位 MB
        :param mailcount: 新邮箱默认最多邮件数
        :param ftpquota: 新邮箱默认网络磁盘空间大小
        :param ftpcount: 新邮箱默认网络磁盘最多文件数
        :param kwargs: 其他可能参数
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            -1  域名数已经达到允许注册数
            1  域名新增失败
            2  相同的域名已经存在
            3  相同的域别名已经存在
            4  相同的 NT 认证域名已经存在
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "domain.added"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def domain_edited(self, domain: str,
                      description: Optional[str] = None,
                      mailquota: Optional[int] = None,
                      mailcount: Optional[int] = None,
                      ftpquota: Optional[int] = None,
                      ftpcount: Optional[int] = None,
                      **kwargs: Optional[Dict]
                      ) -> Dict[str, Any]:
        """ 修改域名

        修改指定域名

        :param domain: 域名
        :param description: 描述
        :param mailquota: 新邮箱默认空间大小,单位 MB
        :param mailcount: 新邮箱默认最多邮件数
        :param ftpquota: 新邮箱默认网络磁盘空间大小
        :param ftpcount: 新邮箱默认网络磁盘最多文件数
        :param kwargs: 其他可能参数
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  域名修改失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "domain.edited"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def domain_delete(self, domain: str) -> Dict[str, Any]:
        """ 删除域名

        删除指定域名

        :param domain: 要删除的域名
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            errno  值  含义
            -1  域名数已经达到允许注册数。
            1  域名删除失败
            2  此域名下存在邮箱，请先删除域下用户，别名和组
            3  主域名不能被删除
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "domain.delete"}
        method_params.update({"domain": domain})

        return self.get_api(**method_params)

    def user(self, domain: str, pageno: Optional[int] = 0) -> Dict[str, Any]:
        """ 获取用户列表

        获取指定域名下用户列表

        :param domain: 域名
        :param pageno: 分页序号，以 0 开始
        :return: 成功返回值
            {
            "result": "ok",
            "info":
                {
                “ users ”: 邮箱用户列表 ,
                “ totalcount ”: 用户总数
                “ pagecount ”:  分页总数
                “ domain ”:  所属域名
                }
            }
        """

        method_params = {"sessid": self.sessid, "method": "user"}
        method_params.update({"domain": domain, "pageno": str(pageno)})

        return self.get_api(**method_params)

    def user_added(self, name: str,
                   domain: str,
                   password: str,
                   authtype: Union[AuthType, int] = Field(AuthType.internal),
                   status: Optional[Union[UserStatus, int]] = Field(UserStatus.normal),
                   fullname: Optional[str] = None,
                   description: Optional[str] = None,
                   homeaddress: Optional[str] = None,
                   homephone: Optional[str] = None,
                   mobile: Optional[str] = None,
                   company: Optional[str] = None,
                   department: Optional[str] = None,
                   jobtitle: Optional[str] = None,
                   office: Optional[str] = None,
                   officephone: Optional[str] = None,
                   mailquota: Optional[int] = None,
                   mailcount: Optional[int] = None,
                   ftpquota: Optional[int] = None,
                   ftpcount: Optional[int] = None,
                   **kwargs: Optional[Dict]
                   ) -> Dict[str, Any]:
        """ 新增用户

        管理员新增邮箱用户接口。

        :param name: 邮箱用户名
        :param domain: 域名
        :param password: 邮箱密码
        :param authtype: 认证方式 0 - 本系统认证； 1 - NT 域认证； 2 - 第三方认证
        :param status: 状态 0 - 正常；1 - 禁止；2 - 等待审核
        :param fullname: 用户姓名
        :param description: 描述
        :param homeaddress: 家庭地址
        :param homephone: 电话电话
        :param mobile: 手机
        :param company: 工作单位
        :param department: 部门
        :param jobtitle: 职位
        :param office: 办公室
        :param officephone: 办公电话
        :param mailquota: 邮箱空间大小 单位MB
        :param mailcount: 邮箱最多邮件数
        :param ftpquota: 网络磁盘空间大小 单位MB
        :param ftpcount: 网络磁盘最多文件数
        :param kwargs: 其他参数
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            -1  用户数已经达到允许注册数
            1  用户新增失败
            2  相同的用户已经存在
            3  相同的用户别名已经存在
            4  相同的邮件组已经存在
            6  此域下用户邮箱数已经达到本域允许的注册数
            7  此域下用户邮箱容量已经达到本域允许的总容量
            8  此域下用户网络磁盘容量已经达到本域允许的总容量
            101  主服务器增加用户失败
            102  主服务器上相同的用户已经存在
            103  主服务器上相同的用户别名已经存在
            104  主服务器上相同的邮件组已经存在
            99  您没有权限进行此项操作
        """
        method_params = {"sessid": self.sessid, "method": "user.added"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def user_edited(self, name: str,
                    domain: str,
                    password: Optional[str] = None,
                    changedpwd: Optional[int] = 0,
                    authtype: Optional[Union[AuthType, int]] = Field(default=AuthType.internal),
                    status: Optional[Union[UserStatus, int]] = Field(default=UserStatus.normal),
                    fullname: Optional[str] = None,
                    description: Optional[str] = None,
                    homeaddress: Optional[str] = None,
                    homephone: Optional[str] = None,
                    mobile: Optional[str] = None,
                    company: Optional[str] = None,
                    department: Optional[str] = None,
                    jobtitle: Optional[str] = None,
                    office: Optional[str] = None,
                    officephone: Optional[str] = None,
                    mailquota: Optional[int] = None,
                    mailcount: Optional[int] = None,
                    ftpquota: Optional[int] = None,
                    ftpcount: Optional[int] = None,
                    **kwargs: Optional[Dict]
                    ) -> Dict[str, Any]:
        """ 修改用户

        管理员修改邮箱用户接口。*如果要修改密码，增加参数 changedpwd ，值设置为 1

        :param name: 邮箱用户名
        :param domain: 域名
        :param password: 邮箱密码
        :param changedpwd: 是否修改密码
        :param authtype: 认证方式 0 - 本系统认证； 1 - NT 域认证； 2 - 第三方认证
        :param status: 状态 0 - 正常；1 - 禁止；2 - 等待审核
        :param fullname: 用户姓名
        :param description: 描述
        :param homeaddress: 家庭地址
        :param homephone: 电话电话
        :param mobile: 手机
        :param company: 工作单位
        :param department: 部门
        :param jobtitle: 职位
        :param office: 办公室
        :param officephone: 办公电话
        :param mailquota: 邮箱空间大小 单位MB
        :param mailcount: 邮箱最多邮件数
        :param ftpquota: 网络磁盘空间大小 单位MB
        :param ftpcount: 网络磁盘最多文件数
        :param kwargs: 其他参数
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  用户修改失败
            99  您没有权限进行此项操作
        """
        method_params = {"sessid": self.sessid, "method": "user.edited"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def user_delete(self, name: str, domain: str) -> Dict[str, Any]:
        """删除用户

        删除邮箱用户
        :param name: 邮箱用户名
        :param domain: 域名
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  用户删除失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "user.delete"}
        method_params.update({"name": name, "domain": domain})

        return self.get_api(**method_params)

    def group(self, domain: str, pageno: Optional[int] = 0) -> Dict[str, Any]:
        """取邮件组列表

        :param domain: 域名
        :param pageno: 页数
        :return: 成功返回值
            {
                "result": "ok",
                "info":
                {
                    “ groups ”: 邮箱用户列表 ,
                    “ totalcount ”: 用户总数
                    “ pagecount ”:  分页总数
                    “ domain ”:  所属域名
                }
            }
        """

        method_params = {"sessid": self.sessid, "method": "group"}
        method_params.update({"pageno": str(pageno), "domain": domain})

        return self.get_api(**method_params)

    def group_added(self,
                    name: str,
                    domain: str,
                    fullname: Optional[str] = None,
                    description: Optional[str] = None,
                    subgroup: Optional[str] = None,
                    members: Optional[str] = None,
                    sendmailright: Optional[int] = 0,
                    sendmailmembers: Optional[str] = None,
                    managers: Optional[str] = None,
                    visibleright: Optional[int] = 0,
                    sendervisible: Optional[bool] = True,
                    **kwargs: Optional[Dict]
                    ) -> Dict[str, Any]:
        """新增邮件组

        :param name: 组名
        :param domain: 域名
        :param fullname: 组名称
        :param description: 描述
        :param subgroup: 子分组，各子分组名以分号(;)分
        :param members: 组成员  各成员名以分号(;)分隔
        :param sendmailright: 发信权限
            0 - 任何人都可以给组员发
            信；
            1 - 只有组员可以发信；
            2 - 仅指定成员可以发信；
            3 - 本域下用户可以发信
        :param sendmailmembers: 发信成员  各发信成员名以分号(;)分隔
        :param managers: 组管理员  各组管理员名以分号(;)分隔
        :param visibleright: 可见权限
            0 - 任何人可以看到此通讯组及其成员；
            1 - 任何人可以看到此通讯组；
            2 - 本域用户可以看到此通讯组及其成员；
            3 - 域用户可以看到此通讯组；
            4 - 组成员可以看到此通讯组及其成员；
            5 - 组成员可以看到此通讯组；
            6 - 只有管理员可以看到此通讯组及成员
        :param sendervisible: 发信成员可见
            0 - 没有特别可见权限；
            1 - 发信成员有相同的可见度。
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  组新增失败
            2  相同的用户已经存在
            3  相同的用户别名已经存在
            4  相同的邮件组已经存在
            101  主服务器增加用户失败
            102  主服务器上相同的用户已经存在
            103  主服务器上相同的用户别名已经存在
            104  主服务器上相同的邮件组已经存在
            99  您没有权限进行此项操作


        """

        method_params = {"sessid": self.sessid, "method": "group.added"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def group_edited(self,
                     name: str,
                     domain: str,
                     fullname: Optional[str] = None,
                     description: Optional[str] = None,
                     subgroup: Optional[str] = None,
                     members: Optional[str] = None,
                     sendmailright: Optional[int] = 0,
                     sendmailmembers: Optional[str] = None,
                     managers: Optional[str] = None,
                     visibleright: Optional[int] = 0,
                     sendervisible: Optional[bool] = True,
                     **kwargs: Optional[Dict]
                     ) -> Dict[str, Any]:
        """新增邮件组

        :param name: 组名
        :param domain: 域名
        :param fullname: 组名称
        :param description: 描述
        :param subgroup: 子分组，各子分组名以分号(;)分
        :param members: 组成员  各成员名以分号(;)分隔
        :param sendmailright: 发信权限
            0 - 任何人都可以给组员发
            信；
            1 - 只有组员可以发信；
            2 - 仅指定成员可以发信；
            3 - 本域下用户可以发信
        :param sendmailmembers: 发信成员  各发信成员名以分号(;)分隔
        :param managers: 组管理员  各组管理员名以分号(;)分隔
        :param visibleright: 可见权限
            0 - 任何人可以看到此通讯组及其成员；
            1 - 任何人可以看到此通讯组；
            2 - 本域用户可以看到此通讯组及其成员；
            3 - 域用户可以看到此通讯组；
            4 - 组成员可以看到此通讯组及其成员；
            5 - 组成员可以看到此通讯组；
            6 - 只有管理员可以看到此通讯组及成员
        :param sendervisible: 发信成员可见
            0 - 没有特别可见权限；
            1 - 发信成员有相同的可见度。
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  组修改失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "group.edited"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def group_addmember(self,
                        name: str,
                        domain: str,
                        groups: str) -> Dict[str, Any]:
        """增加组成员

        :param name: 组成员名
        :param domain: 组成员域名
        :param groups: 组名列表， 多个组以;分隔
        :return:  返回值
            成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  组成员增加失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "group.addmember"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def group_modifymember(self,
                           name: str,
                           domain: str,
                           groups: str) -> Dict[str, Any]:
        """修改组成员

        修改组成员，从系统里的其他组中删除此成员，仅是组名列表中的组成员。

        :param name: 组成员名
        :param domain: 组成员域名
        :param groups: 组名列表， 多个组以;分隔
        :return:  返回值
            成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  组成员增加失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "group.modifymember"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def group_delmember(self,
                        name: str,
                        domain: str,
                        groups: str) -> Dict[str, Any]:
        """删除组成员

        删除组成员，从指的组中删除此成员。

        :param name: 组成员名
        :param domain: 组成员域名
        :param groups: 组名列表， 多个组以;分隔
        :return:  返回值
            成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  组成员增加失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "group.delmember"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def group_delete(self, domain: str, pageno: Optional[int] = 0) -> Dict[str, Any]:
        """
        删除邮件组

        :param domain: 域名
        :param pageno: 页数
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  组删除失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "group.delete"}
        method_params.update({"pageno": str(pageno), "domain": domain})

        return self.get_api(**method_params)

    def adminuser(self, pageno: Optional[int] = 0) -> Dict[str, Any]:
        """管理员列表

        获取管理员列表
        :param pageno:  页码  分页序号，以 0 开始
        :return: 成功返回值
            {
                "result": "ok",
                "info":
                {
                    “ users ”: 用户列表 ,
                    “ totalcount ”: 用户总数
                    “ pagecount ”:  分页总数
                }
            }
        """
        method_params = {"sessid": self.sessid, "method": "adminuser", "pageno": str(pageno)}

        return self.get_api(**method_params)

    def adminuser_added(self,
                        username: str,
                        password: str,
                        description: Optional[str],
                        adminrange: Optional[str],
                        usertype: Optional[int] = 0
                        ) -> Dict[str, Any]:

        """新增 管理员

        :param username: 管理员用户名
        :param password: 管理员密码
        :param description: 描述
        :param adminrange: 允许管理的域 多个域名之间用分号(;)分隔；域管理员需设置
        :param usertype: 管理员类型 0 - 超级管理员；1 - 域管理员
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  管理新增失败
            2  用户名格式不正确
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "adminuser.added"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def adminuser_edited(self,
                         username: str,
                         password: str,
                         description: Optional[str],
                         adminrange: Optional[str],
                         usertype: Optional[int] = 0
                         ) -> Dict[str, Any]:

        """修改管理员

        :param username: 管理员用户名
        :param password: 管理员密码
        :param description: 描述
        :param adminrange: 允许管理的域 多个域名之间用分号(;)分隔；域管理员需设置
        :param usertype: 管理员类型 0 - 超级管理员；1 - 域管理员
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  管理修改失败
            2  用户名格式不正确
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "adminuser.edited"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def adminuser_delete(self,
                         username: str
                         ) -> Dict[str, Any]:
        """删除管理员

        :param username: 管理员用户名
        :return: 成功返回值
            { "result": "ok"}
            失败返回值
            { "result": "err", errno: 1}
            错误代码：
            errno  值  含义
            1  管理删除失败
            99  您没有权限进行此项操作
        """

        method_params = {"sessid": self.sessid, "method": "adminuser.delete", "username": username}

        return self.get_api(**method_params)

    # ######用户端接口开始
    def folders(self):
        """列邮件夹

        列邮件夹
        :return: 返回值
            {
            "result": "ok",
            "info":
                {
                “private”:  个人邮件夹,
                “public”:  公共邮件夹
                “archive”:  归档邮件夹
                “label”:  标签
                }
            }
        """

        method_params = {"sessid": self.sessid, "method": "folders"}

        return self.get_api(**method_params)

    def folders_newfolder(self, newfolder: str) -> Dict[str, Any]:
        """ 新建邮件夹

        新建邮件夹
        :param newfolder: 邮件夹名
        :return: 返回值
            { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "folders.newfolder", "newfolder": newfolder}

        return self.get_api(**method_params)

    def folders_renamefolder(self, optfolder: str, newfolder: str) -> Dict[str, Any]:
        """ 重命名 邮件夹

         重命名 邮件夹
        :param optfolder:
        :param newfolder:
        :return: 返回值
            { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "folders.renamefolder",
                         "newfolder": newfolder, "optfolder": optfolder}

        return self.get_api(**method_params)

    def folders_delfolder(self, optfolder: str) -> Dict[str, Any]:
        """ 删除邮件夹

        删除邮件夹
        :param optfolder: 要操作的邮件夹名
        :return: 返回值
            { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "folders.delfolder", "optfolder": optfolder}

        return self.get_api(**method_params)

    def folders_emptyfolder(self, optfolder: str) -> Dict[str, Any]:
        """ 清空邮件夹

        清空邮件夹
        :param optfolder: 要操作的邮件夹名
        :return: 返回值
            { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "folders.emptyfolder", "optfolder": optfolder}

        return self.get_api(**method_params)

    def msglist(self, folder: str, pag: Optional[int] = 1, **kwargs) -> Dict[str, Any]:
        """列邮件夹的邮件

        列邮件夹的邮件，此接口还有listtype/label参数，以便列出已读未读之类特定邮件。
        | listtype值 |                                       |
        | ---------- | ------------------------------------- |
        | flagged    | 星标邮件                              |
        | read       | 已读                                  |
        | unread     | 未读                                  |
        | answered   | 已回复                                |
        | forwarded  | 已转发                                |
        | label      | 有标签（需要和另一个参数label同时用） |

        :param folder: 文件夹
        :param pag: 页码 分页序号，以 1 开始
        :return: 返回值
        {
        "result": "ok",
        "info":
            {
            “messagelist”:  邮件列表信息,
            “totalpage”:  总分页数
            “newmsg”:  新邮件数
            “msgtotal”:  邮件总数
            }
        }
        """
        method_params = {"sessid": self.sessid, "method": "msglist"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def msglist_delete(self, folder: str, msgid: int) -> Dict[str, Any]:
        """删除邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.delete", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msglist_move(self, folder: str, msgid: int, tofolder: str) -> Dict[str, Any]:
        """删除邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :param tofolder:  目标邮件夹
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.move",
                         "folder": folder, "msgid": str(msgid), "tofolder": tofolder}

        return self.get_api(**method_params)

    def msglist_top(self, folder: str, msgid: int) -> Dict[str, Any]:
        """置顶邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.top", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msglist_untop(self, folder: str, msgid: int) -> Dict[str, Any]:
        """取消置顶邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.untop", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msglist_read(self, folder: str, msgid: int) -> Dict[str, Any]:
        """邮件标记已读

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.read", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msglist_unread(self, folder: str, msgid: int) -> Dict[str, Any]:
        """邮件标记未读

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.unread", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msglist_flag(self, folder: str, msgid: int) -> Dict[str, Any]:
        """设置星标邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.flag", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msglist_unflag(self, folder: str, msgid: int) -> Dict[str, Any]:
        """取消星标邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值{ "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "msglist.unflag", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def msgnum(self) -> Dict[str, Any]:
        """各邮件夹未读邮件数

        :return: 返回值
            {
            "result": "ok",
            "info": 各文件夹的未读邮件数
            }
        """
        method_params = {"sessid": self.sessid, "method": "msgnum"}

        return self.get_api(**method_params)

    def readmsg(self, folder: str, msgid: int) -> Dict[str, Any]:
        """ 阅读邮件

        :param folder: 文件夹
        :param msgid: 邮件标识
        :return: 返回值
            {
            "result": "ok",
            "info":
                {
                “from”:  发件人信息,
                “reply-to”: 回复信息
                “to”:  收件人信息
                “cc”:  抄送信息
                “subject”: 主题
                “date”:  发件日期
                “body”:  信体内容
                “attachment”:  邮件附件
                “memo”:  邮件备注信息
                }
            }
        """
        method_params = {"sessid": self.sessid, "method": "readmsg", "folder": folder, "msgid": str(msgid)}

        return self.get_api(**method_params)

    def newmsg_send(self, to: str,
                    cc: Optional[str],
                    subject: str,
                    msgbody: str,
                    ishtml: Optional[bool],
                    priority: Optional[bool],
                    requestnotify: Optional[bool],
                    **kwargs
                    ) -> Dict[str, Any]:
        """ 发送邮件

        如果要发送带附件的邮件，要先重置写邮件缓存，然后上传附件，最后再发送邮件。重置写邮件缓存，如要发送含附件邮件建议先进行重置操作。
        :param to: 收件人 多个地址用分号(;)分隔
        :param cc: 抄送 多个地址用分号(;)分隔
        :param subject: 邮件主题
        :param msgbody: 邮件信体
        :param ishtml: Html 邮件  0 - 文本邮件；1 – HTML 邮件
        :param priority: 优先级  0 - 正常；1 - 优先
        :param requestnotify: 请求阅读回条 0 - 不需要；1 - 需要
        :return: 返回值 { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "newmsg.send"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue
            if k == "kwargs":
                method_params.update(kwargs)
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def newmsg_reset(self) -> Dict[str, Any]:
        """ 重置写邮件缓存

        :return: 返回值 { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "newmsg.reset"}

        return self.get_api(**method_params)

    def upload_upload(self, attachfile: Union[str, list]) -> Dict[str, Any]:
        """上传附件

        可以一次上传多个文件，也可以分多次上传, 附件文件绝对路径。
        :param attachfile: 附件文件绝对路径或者多附件List。
        :return: 返回值
            {
            "result": "ok",
            “list”:[
                    {
                    "name":"readme.txt",
                    "dispname":"readme.txt",
                    "localname":"C:\\Winmail\\server\\webmail\\temp\\_attachments\\test_baa065a52e385
                    9e4e1a0ee2ca9188713_3a077176425dd745941ee40cd3dc5914",
                    "type":"text/plain",
                    "size":156894
                    },
                    {
                    "name":" 20180705141449.jpg",
                    "dispname":" 20180705141449.jpg",
                    "localname":"C:\\Winmail\\server\\webmail\\temp\\_attachments\\test_baa065a52e385
                    9e4e1a0ee2ca9188713_ 052c4b360ca824741aaa873077e8570d",
                    "type":"image/jpg",
                    "size": 54954
                    }
            ]
            }
        """

        method_params = {"sessid": self.sessid, "method": "upload.upload", "attachfile": attachfile}

        return self.get_api(**method_params)

    def upload_delete(self, attachid: int) -> Dict[str, Any]:
        """删除某个附件

        :param attachid: 附件 ID。上传后返回的附件数组， 所删除附件在数组中序号，序号以 0 开始
        :return: 返回值{ "result": "ok"}
        """

        method_params = {"sessid": self.sessid, "method": "upload.delete", "attachid": attachid}

        return self.get_api(**method_params)

    def addressbook(self, pag: Optional[int] = 1) -> Dict[str, Any]:
        """ 列个人地址簿

        :param pag: 页码分页序号，以 1 开始
        :return: 返回值
                {
                "result": "ok",
                "info":
                    {
                    “group”:  地址组列表 ,
                    “address”:  联系人列表
                    “totalpage”:  总页数
                    “addresscount”:  邮件总数
                    }
                }
        """

        method_params = {"sessid": self.sessid, "method": "addressbook", "pag": str(pag)}

        return self.get_api(**method_params)

    def addressbook_addcontact(self, name: str, email: str,
                               mobile: Optional[str],
                               phone: Optional[str],
                               fax: Optional[str],
                               address: Optional[str],
                               zipcode: Optional[str],
                               company: Optional[str],
                               depart: Optional[str],
                               jobtitle: Optional[str],
                               homephone: Optional[str],
                               homeaddress: Optional[str],
                               homezipcode: Optional[str],
                               im: Optional[str],
                               url: Optional[str],
                               email1: Optional[str],
                               birthday: Optional[str],
                               memo: Optional[str]) -> Dict[str, Any]:
        """ 增加联系人

         增加个人地址簿联系人
        :param name: 姓名
        :param email: 邮件地址
        :param mobile: 手机
        :param phone: 电话
        :param fax: 传真
        :param address: 联系地址
        :param zipcode: 区号
        :param company: 工作单位
        :param depart: 部门
        :param jobtitle: 职位
        :param homephone: 家庭电话
        :param homeaddress: 家庭地址
        :param homezipcode: 家庭区号
        :param im: 即时通信
        :param url: 个人网址
        :param email1: 备份邮箱地址
        :param birthday: 生日
        :param memo: 备注
        :return: 返回值{ "result": "ok"}
        """

        method_params = {"sessid": self.sessid, "method": "addressbook.addcontact"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def addressbook_savecontact(self, id: int, name: str, email: str,
                                mobile: Optional[str],
                                phone: Optional[str],
                                fax: Optional[str],
                                address: Optional[str],
                                zipcode: Optional[str],
                                company: Optional[str],
                                depart: Optional[str],
                                jobtitle: Optional[str],
                                homephone: Optional[str],
                                homeaddress: Optional[str],
                                homezipcode: Optional[str],
                                im: Optional[str],
                                url: Optional[str],
                                email1: Optional[str],
                                birthday: Optional[str],
                                memo: Optional[str]) -> Dict[str, Any]:
        """ 修改 联系人

        :param id: 联系人标识
        :param name: 姓名
        :param email: 邮件地址
        :param mobile: 手机
        :param phone: 电话
        :param fax: 传真
        :param address: 联系地址
        :param zipcode: 区号
        :param company: 工作单位
        :param depart: 部门
        :param jobtitle: 职位
        :param homephone: 家庭电话
        :param homeaddress: 家庭地址
        :param homezipcode: 家庭区号
        :param im: 即时通信
        :param url: 个人网址
        :param email1: 备份邮箱地址
        :param birthday: 生日
        :param memo: 备注
        :return: 返回值{ "result": "ok"}
        """

        method_params = {"sessid": self.sessid, "method": "addressbook.savecontact"}
        for k, v in locals().items():
            if k in ["self", 'k', 'v', 'method_params']:
                continue

            if v is not None:
                method_params.update({k: str(v) if isinstance(v, int) else v})

        return self.get_api(**method_params)

    def addressbook_delecontact(self, id: int) -> Dict[str, Any]:
        """ 删除 联系人

        :param id: 联系人标识
        :return: { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "addressbook.delecontact", "id": str(id)}

        return self.get_api(**method_params)

    def addressbook_addgroup(self, name: str, member: str) -> Dict[str, Any]:
        """增加地址组

        :param name: 分组名称
        :param member: 组成员多个联系人地址用分号(;)分隔
        :return: { "result": "ok"}
        """

        method_params = {"sessid": self.sessid, "method": "addressbook.addgroup",
                         "name": name, "member": member}

        return self.get_api(**method_params)

    def addressbook_savegroup(self, id: int, name: str, member: str) -> Dict[str, Any]:
        """修改地址组

        :param id: 组标识
        :param name: 分组名称
        :param member: 组成员多个联系人地址用分号(;)分隔
        :return: { "result": "ok"}
        """

        method_params = {"sessid": self.sessid, "method": "addressbook.savegroup", "id": str(id),
                         "name": name, "member": member}

        return self.get_api(**method_params)

    def addressbook_delegroup(self, id: int) -> Dict[str, Any]:
        """ 删除地址组

        :param id: 组标识
        :return: { "result": "ok"}
        """
        method_params = {"sessid": self.sessid, "method": "addressbook.delegroup", "id": str(id)}

        return self.get_api(**method_params)

    def netaddressbook(self, pag: Optional[int] = 1) -> Dict[str, Any]:
        """ 列公共地址簿

        :param pag: 页码分页序号，以 1 开始
        :return: 返回值
                {
                "result": "ok",
                "info": {
                    "address": [
                        {
                            "uid": "b",
                            "name": "\u59d3\u540d\u4e59",
                            "email": "b@test.com",
                            "mobile": "13700001111",
                            "company": "\u4e59\u7684\u516c\u53f8\u540d",
                            "department": "\u4e59\u7684\u90e8\u95e8",
                            "jobtitle": " ",
                            "office": " ",
                            "officephone": " ",
                            "homeaddress": " ",
                            "homephone": " ",
                            "chksum": "4c9617b6ca8715663bbed4923c644df0",
                            "usertype": "sys"
                        }
                    ],
                    "totalpage": 2,
                    "addresscount": 31
                }
            }
        """

        method_params = {"sessid": self.sessid, "method": "netaddressbook", "pag": str(pag)}

        return self.get_api(**method_params)

    def systemgroup(self, pag: Optional[int] = 1) -> Dict[str, Any]:
        """ 列系统通信组

        :param pag: 页码分页序号，以 1 开始
        :return: 返回值
        {
            "result": "ok",
            "info": {
                "group": [
                    {
                        "groupid": "everyone",
                        "name": "everyone",
                        "domain": "",
                        "fullname": "everyone@test.com",
                        "description": "",
                        "sendmailright": 3,
                        "visibleright": 0,
                        "sendervisible": 0,
                        "memberlist": "a;a-a;c112;api;1000",
                        "sendmailmember": "",
                        "managerlist": "",
                        "subgroup": [
                            "salse1"
                        ],
                        "extmembername": "",
                        "sortby": 0,
                        "orderno": 9999,
                        "localsend": 0,
                        "fromgroup": 0,
                        "rcptmember": 0,
                        "mailhost": "",
                        "syncaction": "",
                        "synctime": 1694592460,
                        "openid": "",
                        "id": 2982,
                        "groupmail": "everyone@test.com",
                        "manager": 0,
                        "sendmail": 1,
                        "listmember": true,
                        "arraymemberlist": null,
                        "subgrouplevel": 0,
                        "chksum": "0952e1dd7af352500f67524911b21a0c"
                    }
                ],
                "totalpage": 1,
                "groupcount": 7
            }
        }
        """

        method_params = {"sessid": self.sessid, "method": "systemgroup", "pag": str(pag)}

        return self.get_api(**method_params)



# -*- coding:utf-8 -*-

from .MysqlClass import MySqlDb
from .SqliteClass import SqliteDb
from .PostgreClass import PostgreDb
from .Other import start_winmail_service, stop_winmail_service


__all__ = ["MySqlDb", "SqliteDb", "PostgreDb", "start_winmail_service", "stop_winmail_service"]



# -*- coding:utf-8 -*-

import os
import subprocess


def stop_winmail_service():
    """
    停止Winmail服务
    """
    if os.name == "nt":
        subprocess.call("net stop magicwinmailserver", shell=True)
    if os.name == "posix":
        subprocess.call("systemctl stop winmail", shell=True)


def start_winmail_service():
    """
    启动Winmail服务
    """
    if os.name == "nt":
        subprocess.call("net start magicwinmailserver", shell=True)
    if os.name == "posix":
        subprocess.call("systemctl start winmail", shell=True)


# -*- coding:utf-8 -*-

from setuptools import setup, find_packages


# from distutils.core import setup

INSTALL_REQUIRES = [
    'requests',
    'PyMySQL',
    'psycopg2',
    'pydantic',
    "pywin32; sys_platform == 'win32'"
]

setup(
    name='winmail',
    version='2023.11.21',
    python_requires='>=3.6.0',
    description='Winmail API',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='Sway',
    author_email='sway_wang@foxmail.com',
    url='http://winmail.cn',
    packages=find_packages(),
    install_requires=INSTALL_REQUIRES
)



# -*- coding:utf-8 -*-

from winmail.OpenApi import Base, UserStatus, AuthType


class OpenApi12(Base):
    def __init__(self, server: str, port: int, apikey: str, apisecret: str, use_ssl: bool = False) -> None:
        super().__init__(server, port, apikey, apisecret, use_ssl)


if __name__ == "__main__":
    api = OpenApi12('localhost', 80, 'bt66oa8d5b', 'btt87e78f6871aea2016a3916eb65299b7affb18')
    api.login('admin', 'admin', 'admin', 0)
    # print(api.domain())
    print(api.user_edited(name='api', password='test', domain='test.com', authtype=1,
                          status=UserStatus.normal))
    print(api.adminuser())



# -*- coding:utf-8 -*-

from .OpenApi import OpenApi
from .ApiBase import Base, UserStatus, AuthType
from .Api12 import OpenApi12

__all__ = ["OpenApi", "Base", "UserStatus", "AuthType", "OpenApi12"]



# -*- coding: utf-8 -*-

import sqlite3
import os


class SqliteDb:
    """
    dbHandle = SqliteDb(db_file)
    dbHandle.exec(sql)
    dbHandle.fetchone()  or db_handel.fetchall()
    dbHandle.close()
    """
    debug = False

    def __init__(self, db_file=None):
        self.c = None
        self.conn = None
        self.db_file = None
        if db_file is not None:
            self.open(db_file)

    def open(self, db_file):
        if self.debug:
            print(f'db_file: {db_file}')

        if os.path.isfile(db_file):
            self.db_file = os.path.realpath(db_file)
            self.conn = sqlite3.connect(self.db_file)
            self.conn.text_factory = lambda x: str(x, 'utf-8', 'ignore')
            self.c = self.conn.cursor()
            return True
        else:
            return False

    def execute(self, sql):
        # 连接Sqlite执行Sql语言。
        if self.debug:
            print("""class SqliteDb self.DEBUG:\t execute: %s""" % str(sql))
        try:
            self.c.execute(sql)
            return True
        except Exception as e:
            if self.debug:
                print("""class SqliteDb self.DEBUG:\t ERROR: %s""" % str(e))
            return False

    def fetchall(self):

        result = self.c.fetchall()
        if self.debug:
            print(f"fetchall {result}")
        return result

    def fetchone(self):

        result = self.c.fetchone()
        if self.debug:
            print(f"fetchone {result}")
        return result

    def commit(self):
        if self.conn:
            self.conn.commit()

    def close(self):
        if self.conn:
            self.conn.close()
